@@ -98,12 +98,17 @@ CURLcode test(char *URL)
 
   /* PUT style GET_PARAMETERS */
   params = open(libtest_arg2, O_RDONLY);
+  if(params == -1) {
+    fprintf(stderr, "can't open %s\n", libtest_arg2);
+    res = TEST_ERR_MAJOR_BAD;
+    goto test_cleanup;
+  }
   fstat(params, &file_info);
   close(params);
 
   paramsf = fopen(libtest_arg2, "rb");
   if(!paramsf) {
-    fprintf(stderr, "can't open %s\n", libtest_arg2);
+    fprintf(stderr, "can't fopen %s\n", libtest_arg2);
     res = TEST_ERR_MAJOR_BAD;
     goto test_cleanup;
   }
