@@ -101,18 +101,17 @@ static const struct testcase tests[] = {
 UNITTEST_START
 {
   int i;
-  int testnum = sizeof(tests) / sizeof(struct testcase);
   struct Curl_multi *multi = NULL;
   struct Curl_easy *easy = NULL;
   struct curl_slist *list = NULL;
 
-/* important: we setup cache outside of the loop
-  and also clean cache after the loop. In contrast,for example,
-  test 1607 sets up and cleans cache on each iteration. */
+  /* important: we setup cache outside of the loop
+     and also clean cache after the loop. In contrast,for example,
+     test 1607 sets up and cleans cache on each iteration. */
 
-  for(i = 0; i < testnum; ++i) {
+  for(i = 0; i < (int)CURL_ARRAYSIZE(tests); ++i) {
     int j;
-    int addressnum = sizeof (tests[i].address) / sizeof (*tests[i].address);
+    int addressnum = CURL_ARRAYSIZE(tests[i].address);
     struct Curl_addrinfo *addr;
     struct Curl_dns_entry *dns;
     void *entry_id;
